/****************************************************************************
* VCGLib                                                            o o     *
* Visual and Computer Graphics Library                            o     o   *
*                                                                _   O  _   *
* Copyright(C) 2004-2016                                           \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/
/*! \file trimesh_base.cpp
\ingroup code_sample

\brief the minimal example of using the lib

This file contain a minimal example of the library, showing how to load a mesh and how to compute per vertex normals on it.

*/

#include<vcg/complex/complex.h>
#include<wrap/io_trimesh/import_off.h>
#include<wrap/io_trimesh/export_ply.h>
#include<vcg/complex/algorithms/clean.h>
#include<vcg/complex/algorithms/create/platonic.h>

class MyVertex; class MyEdge; class MyFace;
struct MyUsedTypes : public vcg::UsedTypes<vcg::Use<MyVertex>   ::AsVertexType,
                                           vcg::Use<MyEdge>     ::AsEdgeType,
                                           vcg::Use<MyFace>     ::AsFaceType>{};

class MyVertex  : public vcg::Vertex< MyUsedTypes, vcg::vertex::Coord3f, vcg::vertex::Color4b, vcg::vertex::Normal3f, vcg::vertex::BitFlags  >{};
class MyFace    : public vcg::Face<   MyUsedTypes, vcg::face::FFAdj,  vcg::face::VertexRef, vcg::face::BitFlags > {};
class MyEdge    : public vcg::Edge<   MyUsedTypes> {};

class MyMesh    : public vcg::tri::TriMesh< std::vector<MyVertex>, std::vector<MyFace> , std::vector<MyEdge>  > {};
using namespace vcg;

int main( int argc, char **argv )
{
  MyMesh final;
  
  int stepNum = 36;
  float stepAngleDeg = 360/stepNum;
  MyMesh m;
  
  for(int i=0;i<stepNum*3;++i)
  {
    tri::Hexahedron(m);
    Matrix44f T,Trasl,Rot;
    Trasl.SetTranslate(i,0,20.0);
    Rot.SetRotateDeg(i*stepAngleDeg,Point3f(1.0f,0.0f,0.0f));
    T=Rot*Trasl;
    tri::UpdatePosition<MyMesh>::Matrix(m,T);
    tri::Append<MyMesh,MyMesh>::Mesh(final,m);        
  }
  
  vcg::tri::io::ExporterPLY<MyMesh>::Save(final,"final.ply", vcg::tri::io::Mask::IOM_VERTCOLOR);

  // TODO  
  // 1) Try to make a helix torus (e.g wrap the helix above so that 
  //    instead of a infinite cylinder it approximate a torus)
  // 2) build a moebius ring. Scale in a non uniform way the cube so that the section of the ring is a rectangle.
  // 3) (harder) try to actually build a 'correct' thick moebius ring. 
  //    e.g compute and rotate just the four corner of the rectangular section of the ring and 
  //    iteratively add to the base mesh a strip of 8 triangles connecting the tranformed 4 vertexes 
  //    with the last four ones. 
  
  return 0;
}





